"""Stub file for the adjacent functions module.

Note that the default setting behavior is implemented
in each __init__ methods.

Documentation on stubfiles can be found here:
https://mypy.readthedocs.io/en/stable/stubs.html
"""

from typing import Any

from pydantic_core import PydanticUndefined

def Path(
    default: Any = PydanticUndefined,
    *,
    alias: str | None = None,
    title: str | None = None,
    description: str | None = None,
    gt: float | None = None,
    ge: float | None = None,
    lt: float | None = None,
    le: float | None = None,
    min_length: int | None = None,
    max_length: int | None = None,
    pattern: str | None = None,
    example: Any = PydanticUndefined,
    examples: dict[str, Any] | None = None,
    deprecated: bool | None = None,
    include_in_schema: bool = True,
    **extra: Any,
) -> Any: ...
def Query(
    default: Any = PydanticUndefined,
    *,
    alias: str | None = None,
    title: str | None = None,
    description: str | None = None,
    gt: float | None = None,
    ge: float | None = None,
    lt: float | None = None,
    le: float | None = None,
    min_length: int | None = None,
    max_length: int | None = None,
    pattern: str | None = None,
    example: Any = PydanticUndefined,
    examples: dict[str, Any] | None = None,
    deprecated: bool | None = None,
    include_in_schema: bool = True,
    **extra: Any,
) -> Any: ...
def Header(
    default: Any = PydanticUndefined,
    *,
    alias: str | None = None,
    title: str | None = None,
    description: str | None = None,
    gt: float | None = None,
    ge: float | None = None,
    lt: float | None = None,
    le: float | None = None,
    min_length: int | None = None,
    max_length: int | None = None,
    pattern: str | None = None,
    example: Any = PydanticUndefined,
    examples: dict[str, Any] | None = None,
    deprecated: bool | None = None,
    include_in_schema: bool = True,
    convert_underscores: bool = True,
    **extra: Any,
) -> Any: ...
def Cookie(
    default: Any = PydanticUndefined,
    *,
    alias: str | None = None,
    title: str | None = None,
    description: str | None = None,
    gt: float | None = None,
    ge: float | None = None,
    lt: float | None = None,
    le: float | None = None,
    min_length: int | None = None,
    max_length: int | None = None,
    pattern: str | None = None,
    example: Any = PydanticUndefined,
    examples: dict[str, Any] | None = None,
    deprecated: bool | None = None,
    include_in_schema: bool = True,
    **extra: Any,
) -> Any: ...
def Body(
    default: Any = PydanticUndefined,
    *,
    alias: str | None = None,
    title: str | None = None,
    description: str | None = None,
    gt: float | None = None,
    ge: float | None = None,
    lt: float | None = None,
    le: float | None = None,
    min_length: int | None = None,
    max_length: int | None = None,
    pattern: str | None = None,
    example: Any = PydanticUndefined,
    examples: dict[str, Any] | None = None,
    deprecated: bool | None = None,
    include_in_schema: bool = True,
    required: bool = False,
    embed: bool = True,  # for body
    media_type: str = "application/json",
    **extra: Any,
) -> Any: ...
def Form(
    default: Any = PydanticUndefined,
    *,
    alias: str | None = None,
    title: str | None = None,
    description: str | None = None,
    gt: float | None = None,
    ge: float | None = None,
    lt: float | None = None,
    le: float | None = None,
    min_length: int | None = None,
    max_length: int | None = None,
    pattern: str | None = None,
    example: Any = PydanticUndefined,
    examples: dict[str, Any] | None = None,
    deprecated: bool | None = None,
    include_in_schema: bool = True,
    required: bool = False,
    embed: bool = True,
    media_type: str = "application/x-www-form-urlencoded",
    **extra: Any,
) -> Any: ...
def File(  # noqa: N802
    default: Any = PydanticUndefined,
    *,
    alias: str | None = None,
    title: str | None = None,
    description: str | None = None,
    gt: float | None = None,
    ge: float | None = None,
    lt: float | None = None,
    le: float | None = None,
    min_length: int | None = None,
    max_length: int | None = None,
    pattern: str | None = None,
    example: Any = PydanticUndefined,
    examples: dict[str, Any] | None = None,
    deprecated: bool | None = None,
    include_in_schema: bool = True,
    required: bool = False,
    embed: bool = False,
    media_type: str = "multipart/form-data",
    **extra: Any,
) -> Any: ...
